@@ -7,6 +7,7 @@ import sys
 import time
 import warnings
 from copy import deepcopy
+from datetime import datetime
 from random import random
 
 import requests
@@ -20,7 +21,7 @@ else:
     warnings.warn('Module pdpyras will no longer support Python 2.7 as of '
         'June 21, 2021.')
 
-__version__ = '4.1.3'
+__version__ = '4.2.0'
 
 # These are API resource endpoints/methods for which multi-update is supported
 VALID_MULTI_UPDATE_PATHS = [
@@ -80,10 +81,11 @@ def raise_on_error(r):
         return r
     else:
         raise PDClientError("%s %s: API responded with non-success status "
-            "(%d)"%(
+            "(%d): %s" % (
                 r.request.method.upper(),
                 r.request.url.replace('https://api.pagerduty.com', ''),
-                r.status_code
+                r.status_code,
+                r.text[:99]
             ), response=r
         )
 
@@ -632,7 +634,10 @@ class EventsAPISession(PDSession):
     """
     Session class for submitting events to the PagerDuty v2 Events API.
 
-    Provides methods for submitting events to the Events API.
+    Implements methods for submitting events to PagerDuty through the Events API
+    and inherits from :class:`pdpyras.PDSession`.  For more details on usage of
+    this API, refer to the `Events API v2 documentation
+    <https://developer.pagerduty.com/docs/events-api-v2/overview/>`_
 
     Inherits from :class:`PDSession`.
     """
@@ -697,10 +702,7 @@ class EventsAPISession(PDSession):
         if action not in actions:
             raise ValueError("Event action must be one of: "+', '.join(actions))
 
-        event = {
-            'event_action':action,
-            'routing_key': self.api_key
-        }
+        event = {'event_action':action}
 
         event.update(properties)
         if isinstance(dedup_key, string_types):
@@ -716,6 +718,16 @@ class EventsAPISession(PDSession):
                 "deduplication key.", response=response)
         return response_body['dedup_key']
 
+    def post(self, *args, **kw):
+        """
+        Override of ``requests.Session.post``
+
+        Adds the ``routing_key`` parameter to the body before sending.
+        """
+        if 'json' in kw and hasattr(kw['json'], 'update'):
+            kw['json'].update({'routing_key': self.api_key})
+        return super(EventsAPISession, self).post(*args, **kw)
+
     def trigger(self, summary, source, dedup_key=None, severity='critical',
             payload=None, custom_details=None, images=None, links=None):
         """
@@ -779,13 +791,105 @@ class EventsAPISession(PDSession):
             event['links'] = links
         return self.send_event('trigger', dedup_key=dedup_key, **event)
 
+
+class ChangeEventsAPISession(PDSession):
+
+    """
+    Session class for submitting change events to the PagerDuty v2 Change Events API.
+
+    Implements methods for submitting change events to PagerDuty's change events
+    API. See the `Change Events API documentation
+    <https://developer.pagerduty.com/docs/events-api-v2/send-change-events/>`_
+    for more details.
+
+    Inherits from :class:`PDSession`.
+    """
+
+    permitted_methods = ('POST',)
+
+    url = "https://events.pagerduty.com"
+
+    @property
+    def auth_header(self):
+        return {}
+
+    @property
+    def event_timestamp(self):
+        return datetime.utcnow().isoformat()+'Z'
+
+    def prepare_headers(self, method):
+        """Add user agent and content type headers for Change Events API requests."""
+        headers = deepcopy(self.headers)
+        headers.update({
+            'Content-Type': 'application/json',
+            'User-Agent': self.user_agent,
+        })
+        return headers
+
+    def send_change_event(self, **properties):
+        """
+        Send a change event to the v2 Change Events API.
+
+        See: https://developer.pagerduty.com/docs/events-api-v2/send-change-events/
+
+        :param \*\*properties:
+            Properties to set, i.e. ``payload`` and ``links``
+        :returns:
+            The response ID
+        """
+        event = deepcopy(properties)
+        response = self.post('/v2/change/enqueue', json=event)
+        raise_on_error(response)
+        response_body = try_decoding(response)
+        return response_body.get("id", None)
+
+    def submit(self, summary, source=None, custom_details=None, links=None):
+        """
+        Submit an incident change
+
+        :param summary:
+            Summary / brief description of the change.
+        :param source:
+            A human-readable name identifying the source of the change.
+        :param custom_details:
+            The ``payload.custom_details`` property of the payload.
+        :param links:
+            Set the ``links`` property of the event.
+        :type summary: str
+        :type source: str
+        :type custom_details: dict
+        :type links: list
+        :rtype: str
+        """
+        local_var = locals()['custom_details']
+        if not (local_var is None or isinstance(local_var, dict)):
+            raise ValueError("custom_details must be a dict")
+        event = {
+                'routing_key': self.api_key,
+                'payload': {
+                    'summary': summary,
+                    'timestamp': self.event_timestamp,
+                    }
+                }
+        if isinstance(source, str):
+            event['payload']['source'] = source
+        if isinstance(custom_details, dict):
+            event['payload']['custom_details'] = custom_details
+        if links:
+            event['links'] = links
+        return self.send_change_event(**event)
+
+
 class APISession(PDSession):
     """
     Reusable PagerDuty REST API session objects for making API requests.
 
+    Implements features to facilitate usage of the REST API. For more details on
+    how to use each resource type's API, see the `REST API Reference`_.
+
     Includes some convenience functions as well, i.e. :attr:`rget`, :attr:`find`
     and :attr:`iter_all`, to eliminate some repetitive tasks associated with
-    making API calls.
+    API usage.
 
     Inherits from :class:`PDSession`.
 
@@ -892,14 +996,20 @@ class APISession(PDSession):
 
     def find(self, resource, query, attribute='name', params=None):
         """
-        Finds an object of a given resource exactly matching a query.
-
-        Will query a given `resource index`_ endpoint using the ``query``
-        parameter supported by most indexes.
+        Finds an object of a given resource type exactly matching a query.
 
         Returns a dict if a result is found. The structure will be that of an
         entry in the index endpoint schema's array of results. Otherwise, it
-        will return `None` if no result is found or an error is encountered.
+        will return ``None`` if no result is found or an error is encountered.
+
+        Works by querying a given `resource index`_ endpoint using the ``query``
+        parameter. To use this function on any given resource, the resource's
+        index must support the ``query`` parameter; otherwise, the function may
+        not work as expected. If the index ignores the parameter, for instance,
+        this function will take much longer to return; results will not be
+        constrained to those matching the query, and so every result in the
+        index will be downloaded and compared against the query up until a
+        matching result is found or all results have been checked.
 
         :param resource:
             The name of the resource endpoint to query, i.e.
